@@ -35,7 +35,15 @@ import sys
 import time
 from statistics import stdev
 
+import dns.edns
+import dns.exception
 import dns.flags
+import dns.message
+import dns.query
+import dns.rcode
+import dns.rdata
+import dns.rdataclass
+import dns.rdatatype
 import dns.resolver
 import httpx
 
@@ -51,8 +59,8 @@ shutdown = False
 
 def usage():
     print("""%s version %s
-Usage: %s [-46aDeEFhLmqnrvTQxXH] [-i interval] [-w wait] [-p dst_port] [-P src_port] [-S src_ip]
-       %s [-c count] [-t qtype] [-C class] [-s server] hostname
+Usage: %s [-346aDeEFhLmqnrvTQxXHq] [-i interval] [-w wait] [-p dst_port] [-P src_port] [-S src_ip]
+       %s [-c count] [-t qtype] [-C class] [-s server] [--ecs client_subnet] hostname
 
   -h, --help        Show this help message
   -q, --quiet       Suppress output
@@ -63,7 +71,7 @@ Usage: %s [-46aDeEFhLmqnrvTQxXH] [-i interval] [-w wait] [-p dst_port] [-P src_p
   -X, --tls         Use TLS as the transport protocol
   -H, --doh         Use HTTPS as the transport protocol (DoH)
   -3, --http3       Use HTTP/3 as the transport protocol (DoH3)
-  -Q, --doq         Use QUIC as the transport protocol (DoQ)
+  -Q, --quic        Use QUIC as the transport protocol (DoQ)
   -4, --ipv4        Use IPv4 as the network protocol
   -6, --ipv6        Use IPv6 as the network protocol
   -P, --srcport     Specify the source port number for the query (default: 0)
@@ -81,6 +89,7 @@ Usage: %s [-46aDeEFhLmqnrvTQxXH] [-i interval] [-w wait] [-p dst_port] [-P src_p
   -E, --ede         Display EDE (Extended DNS Error) messages, when available
   -n, --nsid        Enable the NSID bit to retrieve resolver identification (implies EDNS)
   -D, --dnssec      Enable the DNSSEC desired flag (implies EDNS)
+      --ecs         Set EDNS Client Subnet option (format: IP/prefix, e.g., 192.168.1.0/24)
   -F, --flags       Display response flags
   -x, --expert      Display additional information (implies --ttl, --flags, --ede)
 """ % (__progname__, __version__, __progname__, ' ' * len(__progname__)))
@@ -147,6 +156,7 @@ def main():
     use_edns = False
     want_nsid = False
     want_dnssec = False
+    client_subnet = None
     show_ttl = False
     force_miss = False
     show_answer = False
@@ -159,7 +169,7 @@ def main():
                                    ["help", "count=", "server=", "quiet", "type=", "wait=", "interval=", "verbose",
                                     "port=", "srcip=", "tcp", "ipv4", "ipv6", "cache-miss", "srcport=", "edns",
                                     "dnssec", "flags", "norecurse", "tls", "doh", "nsid", "ede", "class=", "ttl",
-                                    "expert", "answer", "quic", "http3"])
+                                    "expert", "answer", "quic", "http3", "ecs="])
     except getopt.GetoptError as err:
         # print help information and exit:
         print_stderr(err, False)  # will print something like "option -a not recognized"
@@ -282,6 +292,10 @@ def main():
         elif o in ("-S", "--srcip"):
             src_ip = a
 
+        elif o == "--ecs":
+            client_subnet = a
+            use_edns = True  # ECS requires EDNS
+
         else:
             usage()
 
@@ -319,6 +333,12 @@ def main():
             edns_options = []
             if want_nsid:
                 edns_options.append(dns.edns.GenericOption(dns.edns.NSID, ''))
+            if client_subnet:
+                try:
+                    ecs_option = dns.edns.ECSOption.from_text(client_subnet)
+                    edns_options.append(ecs_option)
+                except Exception as e:
+                    print_stderr("Error: Invalid ECS format '%s': %s" % (client_subnet, e), True)
 
             query = dns.message.make_query(fqdn, rdatatype, rdata_class, flags=request_flags,
                                            use_edns=True, want_dnssec=want_dnssec, payload=1232,
@@ -426,16 +446,37 @@ def main():
                         ans_flags += " --"  # add padding to printer output when dnssec is requested, but AD flag is not set
                     extras += " [%s]" % " ".join([ans_flags, edns_flags]).rstrip(' ')  # show both regular + edns flags
 
-                if want_nsid:
+                # Display EDNS options compactly (only for explicitly enabled options)
+                edns_parts = []
+
+                if want_nsid and answers.options:
                     for ans_opt in answers.options:
                         if ans_opt.otype == dns.edns.OptionType.NSID:
                             nsid_val = ans_opt.nsid
-                            extras += " [ID: %s]" % nsid_val.decode("utf-8")
+                            if nsid_val:
+                                edns_parts.append("NSID:%s" % nsid_val.decode("utf-8"))
 
-                if show_ede:
-                    for ans_opt in answers.options:  # EDE response is optional, but print if there is one
+                if show_ede and answers.options:
+                    for ans_opt in answers.options:
                         if ans_opt.otype == dns.edns.EDE:
-                            extras += " [EDE %d: \"%s\"]" % (ans_opt.code, ans_opt.text or "")
+                            if ans_opt.text:
+                                # Truncate EDE text for ping output to prevent display issues
+                                truncated_text = ans_opt.text[:50] + "..." if len(ans_opt.text) > 50 else ans_opt.text
+                                edns_parts.append("EDE:%d(\"%s\")" % (ans_opt.code, truncated_text))
+                            else:
+                                edns_parts.append("EDE:%d" % ans_opt.code)
+
+                # Always show ECS if present (since it's typically echoed back when requested)
+                if answers.options:
+                    for ans_opt in answers.options:
+                        if ans_opt.otype == dns.edns.OptionType.ECS:
+                            if ans_opt.address:
+                                edns_parts.append("ECS:%s/%d" % (ans_opt.address, ans_opt.scopelen or ans_opt.srclen))
+                            else:
+                                edns_parts.append("ECS:auto")
+
+                if edns_parts:
+                    extras += " [%s]" % ", ".join(edns_parts)
 
                 if show_answer:  # The answer should be displayed at the rightmost
                     for ans in answers.answer:
@@ -449,6 +490,55 @@ def main():
             if verbose:
                 print(answers.to_text(), flush=True)
 
+                # Display EDNS options if present
+                if answers.options:
+                    print(";EDNS OPTIONS")
+                    for ans_opt in answers.options:
+                        option_name = "UNKNOWN"
+                        option_details = ""
+
+                        if ans_opt.otype == dns.edns.OptionType.NSID:
+                            option_name = "NSID"
+                            nsid_val = ans_opt.nsid
+                            option_details = nsid_val.decode("utf-8") if nsid_val else ""
+                        elif ans_opt.otype == dns.edns.OptionType.ECS:
+                            option_name = "ECS"
+                            option_details = "family=%d, source=%d, scope=%d, address=%s" % (
+                                ans_opt.family, ans_opt.srclen, ans_opt.scopelen,
+                                ans_opt.address if ans_opt.address else "None")
+                        elif ans_opt.otype == dns.edns.EDE:
+                            option_name = "EDE"
+                            option_details = "code=%d, text=\"%s\"" % (ans_opt.code, ans_opt.text or "")
+                        elif ans_opt.otype == 10:  # COOKIE
+                            option_name = "COOKIE"
+                            option_details = "length=%d" % len(ans_opt.data)
+                        elif ans_opt.otype == 11:  # TCP-KEEPALIVE
+                            option_name = "TCP-KEEPALIVE"
+                            if len(ans_opt.data) >= 2:
+                                import struct
+                                timeout = struct.unpack('!H', ans_opt.data[:2])[0]
+                                option_details = "timeout=%ds" % timeout
+                        elif ans_opt.otype == 12:  # PADDING
+                            option_name = "PADDING"
+                            option_details = "length=%d" % len(ans_opt.data)
+                        elif ans_opt.otype == 13:  # CHAIN
+                            option_name = "CHAIN"
+                            option_details = "closest_encloser=%s" % ans_opt.data.decode('utf-8', errors='ignore')
+                        elif ans_opt.otype == 14:  # KEY-TAG
+                            option_name = "KEY-TAG"
+                            import struct
+                            if len(ans_opt.data) >= 2:
+                                key_tags = []
+                                for i in range(0, len(ans_opt.data), 2):
+                                    if i + 1 < len(ans_opt.data):
+                                        tag = struct.unpack('!H', ans_opt.data[i:i+2])[0]
+                                        key_tags.append(str(tag))
+                                option_details = "tags=[%s]" % ",".join(key_tags)
+                        else:
+                            option_details = "type=%d, length=%d" % (ans_opt.otype, len(ans_opt.data))
+
+                        print("%s (%d): %s" % (option_name, ans_opt.otype, option_details), flush=True)
+
             time_to_next = (stime + interval) - etime
             if time_to_next > 0:
                 time.sleep(time_to_next)
